unit DPKVMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ToolWin, ComCtrls, ExtCtrls, TeeProcs, TeEngine, Chart, ImgList,
  Menus, ActnList, mmSystem, WaveUtils, WaveStorage, WaveIO, WaveOut, Series,
  WavePlayers, StdCtrls, Spin, ACS_Classes, ACS_Streams, ACS_DXAudio,
  ACS_Audio, ACS_Wave, ACS_Misc;

type
  TfrmDPKV = class(TForm)
    ControlBar1: TControlBar;
    ToolBar1: TToolBar;
    MainMenu1: TMainMenu;
    ImageList: TImageList;
    StatusBar: TStatusBar;
    Panel1: TPanel;
    Panel2: TPanel;
    Chart: TChart;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ActionList: TActionList;
    ToolButton8: TToolButton;
    ToolButton10: TToolButton;
    ActPlay: TAction;
    SamplePopupMenu: TPopupMenu;
    Sample: TLineSeries;
    ActOpenTemplate: TAction;
    ActSaveTemplate: TAction;
    N1: TMenuItem;
    N2: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    N3: TMenuItem;
    N4: TMenuItem;
    N5: TMenuItem;
    N6: TMenuItem;
    ScrollBar: TScrollBar;
    EditCycleCount: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    EditStartFreq: TSpinEdit;
    EditEndFreq: TSpinEdit;
    ActSaveInWAV: TAction;
    CheckSweepFreq: TCheckBox;
    ToolButton12: TToolButton;
    ToolButton4: TToolButton;
    ActStop: TAction;
    ActPause: TAction;
    WaveIn1: TWaveIn;
    SndStream: TStreamIn;
    WaveOut: TWaveOut;
    ProgressBar: TProgressBar;
    WAV1: TMenuItem;
    N7: TMenuItem;
    ActExit: TAction;
    N8: TMenuItem;
    N9: TMenuItem;
    N12: TMenuItem;
    N13: TMenuItem;
    N14: TMenuItem;
    ActNewTemplate: TAction;
    N15: TMenuItem;
    ActNewTemplate1: TMenuItem;
    TrackVolume: TTrackBar;
    AudioOut: TAudioOut;
    Label4: TLabel;
    procedure ActPlayExecute(Sender: TObject);
    procedure ActOpenTemplateExecute(Sender: TObject);
    procedure ActSaveTemplateExecute(Sender: TObject);
    procedure ChartClickSeries(Sender: TCustomChart; Series: TChartSeries;
      ValueIndex: Integer; Button: TMouseButton; Shift: TShiftState; X,
      Y: Integer);
    procedure ChartMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure ChartMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ChartMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ScrollBarScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure ChartScroll(Sender: TObject);
    procedure StockAudioPlayerActivate(Sender: TObject);
    procedure StockAudioPlayerPause(Sender: TObject);
    procedure StockAudioPlayerDeactivate(Sender: TObject);
    procedure StockAudioPlayerResume(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AudioOutThreadException(Sender: TComponent);
    procedure ActStopExecute(Sender: TObject);
    procedure ActSaveInWAVExecute(Sender: TObject);
    procedure ActSaveInWAVUpdate(Sender: TObject);
    procedure AudioOutProgress(Sender: TComponent);
    procedure AudioOutDone(Sender: TComponent);
    procedure ActPauseExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ActExitExecute(Sender: TObject);
    procedure ActNewTemplateExecute(Sender: TObject);
    procedure TrackVolumeChange(Sender: TObject);
    procedure CheckSweepFreqClick(Sender: TObject);
    procedure EditCycleCountChange(Sender: TObject);
    procedure EditStartFreqChange(Sender: TObject);
    procedure EditEndFreqChange(Sender: TObject);
  private
    FDragIndex: integer;
    FDragState: boolean;
    FPaused: boolean;
    FParamChanged: boolean;
    { Private declarations }
    n: double;  //  / /
    CycleCount: integer;
    FStart: integer;
    FEnd: integer;
    FullTime: double;
    Stream: TMemoryStream;
    function CreateSample: boolean;
  public
    { Public declarations }
  end;

var
  frmDPKV: TfrmDPKV;

implementation

{$R *.dfm}

uses
  DialogTemplate;

procedure TfrmDPKV.ActPlayExecute(Sender: TObject);
begin
  SndStream.Loop := CheckSweepFreq.Checked;
  if FParamChanged then
  begin
    if not CreateSample then
      EXIT;
  end;
  Stream.Position := 0;

  ActPlay.Enabled := false;
  if FPaused then
    AudioOut.Resume
  else
    AudioOut.Run;
end;

procedure TfrmDPKV.ActOpenTemplateExecute(Sender: TObject);
var
  F: TFileStream;
  FName: string;
  i: integer;
  A: double;
  Text: TextFile;
begin
  OpenDialog.Filter := '    (*.dpkv)|*.dpkv|  (*.txt)|*.txt';
  OpenDialog.FilterIndex := 1;
  if OpenDialog.Execute then
  begin
    FName := OpenDialog.FileName;
    i := 0;
    case OpenDialog.FilterIndex of
    1:
      begin
        F := TFileStream.Create(FName,fmOpenRead);
        try
          Sample.Clear;
          while F.Position < F.Size do
          begin
            F.ReadBuffer(A,SizeOf(A));
            Sample.AddXY(i,A);
            inc(i);
          end;
        finally
          F.Free;
        end;
      end;
    2:
      begin
        AssignFile(Text,FName);
        Reset(Text);
        Sample.Clear;
        while not EOF(Text) do
        begin
          Readln(Text,A);
          Sample.AddXY(i,A);
          inc(i);
        end;
        CloseFile(Text);
      end;
    end;
    ScrollBar.Max := i-1;
    FParamChanged := true;
  end;
end;

procedure TfrmDPKV.ActSaveTemplateExecute(Sender: TObject);
var
  F: TFileStream;
  FName: string;
  i: integer;
  A: double;
  Text: TextFile;
begin
  SaveDialog.Filter := '    (*.dpkv)|*.dpkv|  (*.txt)|*.txt';
  SaveDialog.FilterIndex := 1;
  SaveDialog.DefaultExt := '.dpkv';
  if SaveDialog.Execute then
  begin
    FName := SaveDialog.FileName;
    i := 0;
    case SaveDialog.FilterIndex of
    1:
      begin
        F := TFileStream.Create(FName,fmCreate);
        try
          for i := 0 to Sample.YValues.Count - 1 do
          begin
            A := Sample.YValue[i];
            F.WriteBuffer(A,SizeOf(A));
          end;
        finally
          F.Free;
        end;
      end;
    2:
      begin
        AssignFile(Text,FName);
        Rewrite(Text);
        for i := 0 to Sample.YValues.Count - 1 do
        begin
          A := Sample.YValue[i];
          Writeln(Text,A);
        end;
        CloseFile(Text);
      end;
    end;
  end;
end;

procedure TfrmDPKV.ChartClickSeries(Sender: TCustomChart;
  Series: TChartSeries; ValueIndex: Integer; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  i: integer;
begin
  FDragState := false;
  if Button = mbLeft then
  begin
    FDragIndex := ValueIndex;
    FDragState := true;
    Screen.Cursor := crCross;
  end;
end;

procedure TfrmDPKV.ChartMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if FDragState then
  begin
    Sample.YValue[FDragIndex] := Sample.YScreenToValue(Y);
  end;
end;

procedure TfrmDPKV.ChartMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if FDragState then
  begin
    Sample.YValue[FDragIndex] := Sample.YScreenToValue(Y);
    FDragState := false;
  end;
  Screen.Cursor := crDefault;
end;

procedure TfrmDPKV.ChartMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbRight then
    Screen.Cursor := crSizeWE;
end;

procedure TfrmDPKV.ScrollBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  if (ScrollBar.Position+100) < Chart.BottomAxis.Minimum then
  begin
    Chart.BottomAxis.Minimum := ScrollBar.Position;
    Chart.BottomAxis.Maximum := ScrollBar.Position+100;
  end
  else
  begin
    Chart.BottomAxis.Maximum := ScrollBar.Position+100;
    Chart.BottomAxis.Minimum := ScrollBar.Position;
  end
end;

procedure TfrmDPKV.ChartScroll(Sender: TObject);
begin
  ScrollBar.Position := Round(Chart.BottomAxis.Minimum/Sample.YValues.Count*ScrollBar.Max);
end;

function TfrmDPKV.CreateSample: boolean;
const
  StatTemplate = '                 ';
var
  dt: double;
  dF: double;
  dS: double;
  S: double;
  IntS: integer;
  SamplePoint: integer;
  T: double;
  F: double;
  Nq: integer;
  A: double;
  U: smallint;
  i: integer;
  MemData: TMemoryStream;
  tmpCursor: TCursor;
  StatusIdx: Byte;
  StatusSign: integer;
  Sweep: boolean;
begin
  result := false;
  FPaused := false;
  Sweep := CheckSweepFreq.Checked;
  Nq := Sample.YValues.Count;
  if Nq <= 0 then
  begin
    MessageDlg('  !',mtWarning,[mbOk],0);
    EXIT;
  end;
  tmpCursor := Screen.Cursor;
  ActPlay.Enabled := false;
  Application.ProcessMessages;
  try
    Screen.Cursor := crHourGlass;
    StatusBar.SimpleText := ' ';
    CycleCount := EditCycleCount.Value;
    FStart := EditStartFreq.Value;
    FEnd := EditEndFreq.Value;
    FullTime := CycleCount*2*60/(FStart + FEnd);
    dt := 1/44100;
    dF := (FEnd - FStart)/(FullTime/dt);
    if Sweep then
      dF := 2*dF;
    F := FStart;
    T := 0;
    S := 0;
    i := 0;
    StatusIdx := 0;
    StatusSign := 1;
    Stream.Clear;
    repeat
      inc(i);
      T := T + dt;
      if (T > FullTime/2) AND Sweep then
        F := F - dF
      else
        F := F + dF;
      dS := Nq*dt*F/60;
      S := (Nq-1)*Frac((S + dS)/(Nq-1));
      IntS := Trunc(S);
      A := Sample.YValue[IntS] + (Sample.YValue[IntS+1] - Sample.YValue[IntS])*(S-IntS);
      U := Round(A/2*32000);
      Stream.Write(U,SizeOf(U));

      if (i mod 40000) = 0 then
      begin
        if StatusIdx > 16 then
          StatusSign := -1;
        if StatusIdx < 1 then
          StatusSign := 1;
        StatusIdx := StatusIdx + StatusSign;

        StatusBar.SimpleText := ' '+Copy(StatTemplate,1,StatusIdx)+'|';
      end;

    until T > FullTime;
    StatusBar.SimpleText := '';
    StatusBar.SimpleText := 'C   ';
    SndStream.Stream := Self.Stream;
    FParamChanged := false;
    result := true;
  finally
    Screen.Cursor := tmpCursor;
    ActPlay.Enabled := true;
  end;
end;

procedure TfrmDPKV.StockAudioPlayerActivate(Sender: TObject);
begin
  StatusBar.SimpleText := '';
end;

procedure TfrmDPKV.StockAudioPlayerPause(Sender: TObject);
begin
  StatusBar.SimpleText := '';
end;

procedure TfrmDPKV.StockAudioPlayerDeactivate(Sender: TObject);
begin
  StatusBar.SimpleText := '';
end;

procedure TfrmDPKV.StockAudioPlayerResume(Sender: TObject);
begin
  StatusBar.SimpleText := '';
end;

procedure TfrmDPKV.FormCreate(Sender: TObject);
begin
  Stream := TMemoryStream.Create;
  SndStream.Stream := Self.Stream;
  FParamChanged := true;
end;

procedure TfrmDPKV.FormDestroy(Sender: TObject);
begin
  Stream.Free;
end;

procedure TfrmDPKV.AudioOutThreadException(Sender: TComponent);
begin
  ShowMessage('Uknown error in tread!');
end;

procedure TfrmDPKV.ActStopExecute(Sender: TObject);
begin
  AudioOut.Stop();
end;

procedure TfrmDPKV.ActSaveInWAVExecute(Sender: TObject);
begin
  if FParamChanged then
  begin
    if not CreateSample then
    begin
      ShowMessage('  ,  !');
      EXIT;
    end;
  end;
  Stream.Position := 0;

  //if SndStream.Loop then
  //  MessageDlg('   !       !',mtWarning,[mbOk],0);
  SaveDialog.Filter := ' WAV (*.wav)|*.wav';
  SaveDialog.FilterIndex := 1;
  SaveDialog.DefaultExt := '.wav';
  SaveDialog.FileName := 'dpkv';
  if SaveDialog.Execute then
  begin
    WaveOut.FileName := SaveDialog.FileName;
    SndStream.Loop := false;
    Stream.Position := 0;
    WaveOut.Run;
    SndStream.Loop := CheckSweepFreq.Checked;
  end;
end;

procedure TfrmDPKV.ActSaveInWAVUpdate(Sender: TObject);
begin
  //ActSaveInWAV.Enabled := Stream.Size > 0;
end;

procedure TfrmDPKV.AudioOutProgress(Sender: TComponent);
begin
  ProgressBar.Position := Round(SndStream.Position*100/SndStream.Size);
end;

procedure TfrmDPKV.AudioOutDone(Sender: TComponent);
begin
  ActPlay.Enabled := true;
  ActStop.Enabled := true;
  ActPause.Enabled := true;
end;

procedure TfrmDPKV.ActPauseExecute(Sender: TObject);
begin
  AudioOut.Pause;
  FPaused := true;
  ActPlay.Enabled := true;
end;

procedure TfrmDPKV.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  AudioOut.Stop(false);
end;

procedure TfrmDPKV.ActExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmDPKV.ActNewTemplateExecute(Sender: TObject);
var
  Dlg: TDlgTemplate;
  NPoint, NPeriod: integer;
  i: integer;
  U: double;
begin
  Dlg := TDlgTemplate.Create(Self);
  try
    if Dlg.ShowModal = mrOk then
    begin
      NPoint := Dlg.EditNPoint.Value;
      NPeriod := Dlg.EditNPeriod.Value;
      i := 0;
      Sample.Clear;
      if Dlg.SinTemplate.Checked then
      begin
        repeat
          U := sin(2*Pi*(i mod NPoint)/NPoint);
          Sample.AddXY(i,U);
          inc(i);
        until i > NPeriod*NPoint;
      end
      else if Dlg.SawToothTemplate.Checked then
      begin
        repeat
          if (i mod NPoint) < (NPoint/2) then
            U := (i mod NPoint)/(NPoint/4) - 1
          else
            U := 1 - (i mod NPoint - NPoint/2)/(NPoint/4);
          Sample.AddXY(i,U);
          inc(i);
        until i > NPeriod*NPoint;
      end
      else if Dlg.MeanderTemplate.Checked then
      begin
        repeat
          if (i mod NPoint) < (NPoint/2) then
            U := 1
          else
            U := -1;
          Sample.AddXY(i,U);
          inc(i);
        until i > NPeriod*NPoint;
      end;
      FParamChanged := true;
    end;
  finally
    Dlg.Release;
  end;
end;

procedure TfrmDPKV.TrackVolumeChange(Sender: TObject);
begin
  AudioOut.Volume := Round((100 - TrackVolume.Position)*2.5);
  TrackVolume.SelStart := TrackVolume.Position;
end;

procedure TfrmDPKV.CheckSweepFreqClick(Sender: TObject);
begin
  SndStream.Loop := CheckSweepFreq.Checked;
  FParamChanged := true;
end;

procedure TfrmDPKV.EditCycleCountChange(Sender: TObject);
begin
  FParamChanged := true;
end;

procedure TfrmDPKV.EditStartFreqChange(Sender: TObject);
begin
  FParamChanged := true;
end;

procedure TfrmDPKV.EditEndFreqChange(Sender: TObject);
begin
  FParamChanged := true;
end;

initialization
  if waveOutGetNumDevs = 0 then
    ShowMessage('     !');
end.
